( cd opt/kde/lib/kde3 ; rm -rf libkmailpart.so )
( cd opt/kde/lib/kde3 ; ln -sf libkmailpart.so.1.0.0 libkmailpart.so )
( cd opt/kde/lib/kde3 ; rm -rf libkmailpart.so.1 )
( cd opt/kde/lib/kde3 ; ln -sf libkmailpart.so.1.0.0 libkmailpart.so.1 )
( cd opt/kde/lib/kde3 ; rm -rf libknodepart.so )
( cd opt/kde/lib/kde3 ; ln -sf libknodepart.so.1.0.0 libknodepart.so )
( cd opt/kde/lib/kde3 ; rm -rf libknodepart.so.1 )
( cd opt/kde/lib/kde3 ; ln -sf libknodepart.so.1.0.0 libknodepart.so.1 )
( cd opt/kde/lib ; rm -rf libkabinterfaces.so )
( cd opt/kde/lib ; ln -sf libkabinterfaces.so.1.0.0 libkabinterfaces.so )
( cd opt/kde/lib ; rm -rf libkabinterfaces.so.1 )
( cd opt/kde/lib ; ln -sf libkabinterfaces.so.1.0.0 libkabinterfaces.so.1 )
( cd opt/kde/lib ; rm -rf libkaddressbook.so )
( cd opt/kde/lib ; ln -sf libkaddressbook.so.0.0.0 libkaddressbook.so )
( cd opt/kde/lib ; rm -rf libkaddressbook.so.0 )
( cd opt/kde/lib ; ln -sf libkaddressbook.so.0.0.0 libkaddressbook.so.0 )
( cd opt/kde/lib ; rm -rf libkalarmd.so )
( cd opt/kde/lib ; ln -sf libkalarmd.so.0.0.0 libkalarmd.so )
( cd opt/kde/lib ; rm -rf libkalarmd.so.0 )
( cd opt/kde/lib ; ln -sf libkalarmd.so.0.0.0 libkalarmd.so.0 )
( cd opt/kde/lib ; rm -rf libkcal.so )
( cd opt/kde/lib ; ln -sf libkcal.so.2.0.0 libkcal.so )
( cd opt/kde/lib ; rm -rf libkcal.so.2 )
( cd opt/kde/lib ; ln -sf libkcal.so.2.0.0 libkcal.so.2 )
( cd opt/kde/lib ; rm -rf libkdenetwork.so )
( cd opt/kde/lib ; ln -sf libkdenetwork.so.2.2.0 libkdenetwork.so )
( cd opt/kde/lib ; rm -rf libkdenetwork.so.2 )
( cd opt/kde/lib ; ln -sf libkdenetwork.so.2.2.0 libkdenetwork.so.2 )
( cd opt/kde/lib ; rm -rf libkdepim.so )
( cd opt/kde/lib ; ln -sf libkdepim.so.1.0.0 libkdepim.so )
( cd opt/kde/lib ; rm -rf libkdepim.so.1 )
( cd opt/kde/lib ; ln -sf libkdepim.so.1.0.0 libkdepim.so.1 )
( cd opt/kde/lib ; rm -rf libkdgantt.so )
( cd opt/kde/lib ; ln -sf libkdgantt.so.0.0.0 libkdgantt.so )
( cd opt/kde/lib ; rm -rf libkdgantt.so.0 )
( cd opt/kde/lib ; ln -sf libkdgantt.so.0.0.0 libkdgantt.so.0 )
( cd opt/kde/lib ; rm -rf libkgantt.so )
( cd opt/kde/lib ; ln -sf libkgantt.so.0.0.2 libkgantt.so )
( cd opt/kde/lib ; rm -rf libkgantt.so.0 )
( cd opt/kde/lib ; ln -sf libkgantt.so.0.0.2 libkgantt.so.0 )
( cd opt/kde/lib ; rm -rf libkmailprivate.so )
( cd opt/kde/lib ; ln -sf libkmailprivate.so.0.0.0 libkmailprivate.so )
( cd opt/kde/lib ; rm -rf libkmailprivate.so.0 )
( cd opt/kde/lib ; ln -sf libkmailprivate.so.0.0.0 libkmailprivate.so.0 )
( cd opt/kde/lib ; rm -rf libknewstuff.so )
( cd opt/kde/lib ; ln -sf libknewstuff.so.1.0.0 libknewstuff.so )
( cd opt/kde/lib ; rm -rf libknewstuff.so.1 )
( cd opt/kde/lib ; ln -sf libknewstuff.so.1.0.0 libknewstuff.so.1 )
( cd opt/kde/lib ; rm -rf libkontact.so )
( cd opt/kde/lib ; ln -sf libkontact.so.1.0.0 libkontact.so )
( cd opt/kde/lib ; rm -rf libkontact.so.1 )
( cd opt/kde/lib ; ln -sf libkontact.so.1.0.0 libkontact.so.1 )
( cd opt/kde/lib ; rm -rf libkorganizer.so )
( cd opt/kde/lib ; ln -sf libkorganizer.so.1.0.0 libkorganizer.so )
( cd opt/kde/lib ; rm -rf libkorganizer.so.1 )
( cd opt/kde/lib ; ln -sf libkorganizer.so.1.0.0 libkorganizer.so.1 )
( cd opt/kde/lib ; rm -rf libkorganizer_eventviewer.so )
( cd opt/kde/lib ; ln -sf libkorganizer_eventviewer.so.1.0.0 libkorganizer_eventviewer.so )
( cd opt/kde/lib ; rm -rf libkorganizer_eventviewer.so.1 )
( cd opt/kde/lib ; ln -sf libkorganizer_eventviewer.so.1.0.0 libkorganizer_eventviewer.so.1 )
( cd opt/kde/lib ; rm -rf libkpilot.so )
( cd opt/kde/lib ; ln -sf libkpilot.so.0.0.0 libkpilot.so )
( cd opt/kde/lib ; rm -rf libkpilot.so.0 )
( cd opt/kde/lib ; ln -sf libkpilot.so.0.0.0 libkpilot.so.0 )
( cd opt/kde/lib ; rm -rf libkpimexchange.so )
( cd opt/kde/lib ; ln -sf libkpimexchange.so.1.0.0 libkpimexchange.so )
( cd opt/kde/lib ; rm -rf libkpimexchange.so.1 )
( cd opt/kde/lib ; ln -sf libkpimexchange.so.1.0.0 libkpimexchange.so.1 )
( cd opt/kde/lib ; rm -rf libkpinterfaces.so )
( cd opt/kde/lib ; ln -sf libkpinterfaces.so.1.0.0 libkpinterfaces.so )
( cd opt/kde/lib ; rm -rf libkpinterfaces.so.1 )
( cd opt/kde/lib ; ln -sf libkpinterfaces.so.1.0.0 libkpinterfaces.so.1 )
( cd opt/kde/lib ; rm -rf libksieve.so )
( cd opt/kde/lib ; ln -sf libksieve.so.0.0.0 libksieve.so )
( cd opt/kde/lib ; rm -rf libksieve.so.0 )
( cd opt/kde/lib ; ln -sf libksieve.so.0.0.0 libksieve.so.0 )
( cd opt/kde/lib ; rm -rf libksync.so )
( cd opt/kde/lib ; ln -sf libksync.so.1.0.0 libksync.so )
( cd opt/kde/lib ; rm -rf libksync.so.1 )
( cd opt/kde/lib ; ln -sf libksync.so.1.0.0 libksync.so.1 )
( cd opt/kde/lib ; rm -rf libktnef.so )
( cd opt/kde/lib ; ln -sf libktnef.so.1.0.0 libktnef.so )
( cd opt/kde/lib ; rm -rf libktnef.so.1 )
( cd opt/kde/lib ; ln -sf libktnef.so.1.0.0 libktnef.so.1 )
( cd opt/kde/lib ; rm -rf libmimelib.so )
( cd opt/kde/lib ; ln -sf libmimelib.so.1.0.1 libmimelib.so )
( cd opt/kde/lib ; rm -rf libmimelib.so.1 )
( cd opt/kde/lib ; ln -sf libmimelib.so.1.0.1 libmimelib.so.1 )
( cd opt/kde/share/doc/HTML/en/kaddressbook ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kaddressbook ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kalarm ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kalarm ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kandy ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kandy ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/karm ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/karm ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kcontrol/kalarmd ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kcontrol/kalarmd ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kdepim-apidocs ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kdepim-apidocs ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kgpgcertmanager ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kgpgcertmanager ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kmail ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kmail ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/knode ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/knode ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/knotes ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/knotes ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/konsolekalendar ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/konsolekalendar ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kontact ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kontact ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/korganizer ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/korganizer ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/korn ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/korn ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kpilot ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kpilot ; ln -sf /opt/kde/share/doc/HTML/en/common common )
