<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="kpilot" version="2">
<MenuBar>
  <Menu name="file" noMerge="1"><text>&amp;File</text>
    <Action name="file_hotsync" />
    <Action name="file_fastsync" />
    <Action name="file_list" />
    <Action name="file_backup" />
    <Action name="file_restore" />
    <Separator />
    <Action name="file_quit" />
  </Menu>
  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="options_show_statusbar" />
    <Action name="options_show_toolbar" />
    <Merge name="StandardToolBarMenuHandler" />
    <Separator />
    <Action name="options_configure_conduits" />
    <Action name="options_configure_keybinding" />
    <Action name="options_configure_toolbars" />
    <Action name="options_configure" />
  </Menu>
</MenuBar>

<Menu name="rb_popup">
  <Action name="file_hotsync" />
  <Action name="file_fastsync" />
  <Action name="file_backup" />
  <Action name="file_restore" />
  <Separator />
  <Action name="file_quit"/>
</Menu>

<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="file_hotsync" />
  <Action name="file_backup" />
  <Separator />
  <Action name="file_quit" />
</ToolBar>

</kpartgui>


<!--
Local Variables:
mode: xml
sgml-omittag: nil
sgml-shorttag: t
sgml-general-insert-case: lower
End:
-->

