<!DOCTYPE kpartgui>
<kpartgui name="kaddressbook" version="13">

  <MenuBar>
    <Menu name="file" noMerge="1"><text>&amp;File</text>
      <Action name="file_new_contact"/>
      <Action name="file_properties"/>
      <Separator/>
      <Action name="file_sync"/>
      <Menu name="file_import"><text>&amp;Import</text>
      </Menu>
      <Menu name="file_export"><text>&amp;Export</text>
      </Menu>
      <Separator/>
      <Action name="file_print"/>
      <Action name="file_mail"/>
      <Action name="file_mail_vcard"/>
      <Separator/>
      <Action name="file_quit"/>
    </Menu>

    <Menu name="edit" noMerge="1"><text>&amp;Edit</text>
      <Action name="edit_undo"/>
      <Action name="edit_redo"/> 
      <Separator/>
      <Action name="edit_cut"/>
      <Action name="edit_copy"/>
      <Action name="edit_paste"/>
      <Separator/>
      <Action name="edit_delete" append="edit_paste_merge"/>
      <Action name="edit_select_all"/>
      <Action name="edit_set_categories"/>
      <Action name="edit_set_personal"/>
    </Menu>

    <Menu name="view" noMerge="1"><text>&amp;View</text>
      <Action name="select_view"/>
      <Action name="view_add"/>
      <Action name="view_modify"/>
      <Action name="view_delete"/>
      <Separator/>
      <Action name="view_refresh"/>
    </Menu>

    <Menu name="tools" noMerge="1"><text>Tools</text>
      <Action name="ldap_lookup"/>
    </Menu>

    <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
      <Merge name="StandardToolBarMenuHandler" />
      <Action name="options_show_extensions"/>
      <Action name="options_show_jump_bar"/>
      <Action name="options_show_details"/>
      <Action name="options_show_statusbar"/>
      <Merge name="show_merge"/>
      <Separator/>
      <Merge/>
      <Action name="options_edit_filters"/>
      <Separator/>
      <Action name="kaddressbook_configure" group="settings_configure"/>
    </Menu>

  </MenuBar>

  <Menu name="RMBPopup">
    <Action name="file_properties"/>
    <Action name="file_new_contact"/>
    <Action name="edit_delete"/>
    <Separator/>
    <Action name="file_mail"/>
    <Action name="file_mail_vcard"/>
  </Menu>

  <ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
    <Action name="file_new_contact"/>
    <Action name="file_properties"/>
    <Separator/>
    <Action name="file_sync" />
    <Action name="file_print" />
    <Separator/>
    <Action name="file_mail"/>
    <Separator/>
    <Action name="ldap_lookup"/>
  </ToolBar>

  <ToolBar name="addresseeToolBar" noMerge="1"><text>Contact Toolbar</text>
    <Action name="edit_undo"/>
    <Action name="edit_redo"/> 
    <Separator/>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="edit_paste"/>
    <Separator/>
    <Action name="edit_delete"/>
  </ToolBar>

</kpartgui>
