<!DOCTYPE kpartgui>
<kpartgui name="KNReader" version="12">

<MenuBar>
  <Menu noMerge="1" name="file"><text>&amp;File</text>
    <Action name="file_save"/>
    <Action name="file_print"/>
    <Separator/>
    <Action name="file_close"/>
  </Menu>
  <Menu noMerge="1" name="edit"><text>&amp;Edit</text>
    <Action name="edit_copy"/>
    <Action name="edit_select_all"/>
  </Menu>
  <Menu noMerge="1" name="view"><text>&amp;View</text>
    <Action name="view_showAllHdrs"/>
    <Action name="view_rot13"/>
    <Action name="article_verify"/>
    <Action name="view_useFixedFont"/>
    <Action name="set_charset"/>
  </Menu>
  <Menu name="article"><text>&amp;Article</text>
    <Action name="article_postReply"/>
    <Action name="article_mailReply"/>
    <Action name="article_forward"/>
    <Separator/>
    <Action name="article_cancel"/>
    <Action name="article_supersede"/>
    <Separator/>
    <Action name="article_viewSource"/>
  </Menu>
</MenuBar>

<ToolBar noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
  <Action name="file_save"/>
  <Action name="file_print"/>
  <Action name="edit_copy"/>
  <Separator/>
  <Action name="article_postReply"/>
  <Action name="article_mailReply"/>
  <Action name="article_forward"/>
</ToolBar>

<Menu name="body_popup">
  <Action name="file_save"/>
  <Action name="file_print"/>
  <Separator/>
  <Action name="edit_copy"/>
  <Action name="edit_select_all"/>
  <Separator/>
  <Action name="view_showAllHdrs"/>
  <Action name="article_viewSource"/>
  <Separator/>
  <Action name="view_rot13"/>
  <Action name="article_verify"/>
  <Separator/>
  <Action name="view_useFixedFont"/>
  <Action name="set_charset"/>
</Menu>

</kpartgui>
