<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<kpartgui name="KNotes" version="2">

<Menu name="knotes_context" noMerge="1"><text>KNotes</text>
    <title icon="knotes">KNotes</title>
    <Action name="new_note"/>
    <Action name="new_note_clipboard"/>
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure"/>
    <Separator/>
    <Menu name="help" icon="help" noMerge="1"><text>&amp;Help</text>
        <Action name="help_contents"/>
        <Separator/>
        <Action name="help_report_bug"/>
        <Separator/>
        <Action name="help_about_app"/>
        <Action name="help_about_kde"/>
    </Menu>
    <Separator/>
    <Action name="file_quit"/>
</Menu>

<Menu name="notes_menu" noMerge="1"><text>Notes</text>
    <title>Notes</title>
    <ActionList name="notes"/>
</Menu>

<Menu name="note_context" noMerge="1">
    <Action name="insert_date"/>
    <Action name="mail_note"/>
    <Action name="print_note"/>
    <Action name="configure_note"/>
    <Separator/>
    <Action name="new_note"/>
    <Action name="rename_note"/>
    <Action name="hide_note"/>
    <Action name="delete_note"/>
    <Separator/>
    <Action name="to_desktop"/>
    <Action name="always_on_top"/>
</Menu>

<Menu name="note_edit" noMerge="1">
    <Action name="edit_undo"/>
    <Action name="edit_redo"/>
    <Separator/>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="edit_paste"/>
    <Action name="edit_clear"/>
    <Separator/>
    <Action name="edit_select_all"/>
</Menu>

</kpartgui>
