<!DOCTYPE kpartgui >
<kpartgui version="15" name="kontact" >
<MenuBar>
  <Menu name="file" noMerge="1">
    <text>&amp;File</text>
	<Merge/>
	<Separator/>
    <Action name="action_new"/>
	<Separator/>
    <Action name="file_quit"/>
	<Merge/>
  </Menu>
  <Merge />
  <Menu name="settings">
     <text>&amp;Settings</text>
     <Merge append="save_merge"/>
<!-- <Separator/> -->
     <DefineGroup name="settings_configure" append="configure_merge"/>
     <Action name="settings_configure_kontact" append="configure_merge"/>

<!-- Those actions have to be set by the parts because some applications
     have app.rc == part.rc 
-->
<!--
     <Action name="options_configure_keybinding"/>
     <Action name="options_configure_toolbars"/>
-->
  </Menu>
  <Menu name="help"><text>&amp;Help</text>
     <Action name="help_tipofday"/>
  </Menu>
</MenuBar>
<ToolBar position="Top" iconText="IconOnly" noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
  <Action name="action_new"/>
  <Merge/>
  <Action name="help_whats_this"/>
</ToolBar>
</kpartgui>
