<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kandy" version="2">
<MenuBar>
  <Menu name="show"><text>&amp;Show</text>
    <Action name="show_mobilegui" />
  </Menu>
  <Menu name="modem"><text>&amp;Modem</text>
    <Action name="modem_connect" />
    <Action name="modem_disconnect" />
  </Menu>
</MenuBar>
</kpartgui>
