<!DOCTYPE kpartgui ><kpartgui name="kgpgcertmanager" version="1" >
<MenuBar>
	<Menu name="file" >
		<text>File</text>
		<Action name="redisplay" />
		<Action name="quit" />
	</Menu>
	<Menu name="view" >
		<text>View</text>
		<Action name="viewCRLs" />
	</Menu>
	<Menu name="certMenu" >
		<text>Certificates</text>
		<Action name="newCert" />
		<Action name="revokeCert" />
		<Action name="extendCert" />
	</Menu>
	<Menu name="importMenu">
	        <text>Import</text>
		<Action name="importCertFromFile" />		        
		<Action name="importCRLFromFile" />
	</Menu>
</MenuBar>

<ToolBar alreadyVisited="1" position="Top" iconText="IconOnly" noMerge="1" name="mainToolBar" >
<text>Main Toolbar</text>
	<Action name="label_action" />
	<Action name="query_lineedit_action" />
	<Action name="location_combo_action" />
	<Action name="find" />
</ToolBar>

</kpartgui>
